/*-----------------------------------------------------------------------------
 * Name:    ADC_TWR-KM34Z50MV3.c
 * Purpose: A/D Converter interface for TWR-KM34Z50MV3 evaluation board
 * Rev.:    1.00
 *----------------------------------------------------------------------------*/

/* Copyright (c) 2015 ARM LIMITED

   All rights reserved.
   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions are met:
   - Redistributions of source code must retain the above copyright
     notice, this list of conditions and the following disclaimer.
   - Redistributions in binary form must reproduce the above copyright
     notice, this list of conditions and the following disclaimer in the
     documentation and/or other materials provided with the distribution.
   - Neither the name of ARM nor the names of its contributors may be used
     to endorse or promote products derived from this software without
     specific prior written permission.
   *
   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
   AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
   ARE DISCLAIMED. IN NO EVENT SHALL COPYRIGHT HOLDERS AND CONTRIBUTORS BE
   LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
   CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
   SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
   INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
   CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
   ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
   POSSIBILITY OF SUCH DAMAGE.
   ---------------------------------------------------------------------------*/

#include "MKM34ZA5.h"                   // Device header
#include "Board_ADC.h"                  // ::Board Support:A/D Converter

#define ADC_RESOLUTION        16        // Number of A/D converter bits

static volatile uint16_t AD_last;       // Last converted value
static volatile uint8_t  AD_done;       // AD conversion done flag

/**
  \fn          int32_t ADC_Initialize (void)
  \brief       Initialize Analog-to-Digital Converter
  \returns
   - \b  0: function succeeded
   - \b -1: function failed
*/
int32_t ADC_Initialize (void) {

  SIM->SCGC6 |=  (1U << SIM_SCGC6_ADC_SHIFT);        // enable ADC clock

  ADC->CFG1   = ((0U << 7) |                         // normal power configuration
                 (3U << 5) |                         // input clock / 8
                 (1U << 4) |                         // long sample time
                 (3U << 2) |                         // single-ended 16-bit conversion
                 (0U     )  );                       // Bus clock
  ADC->CFG2   = ((0U << 4) |                         // ADxxa channels are selected
                 (0U << 3) |                         // Asynchronous clock output disabled
                 (1U << 2) |                         // High-speed conversion sequence selected
                 (0U << 0)  );                       // Default longest sample time

  ADC->CV1    =   0U;
  ADC->CV2    =   0U;
  ADC->SC2    = ((0U << 6) |                         // Software trigger selected
                 (0U << 0)  );                       // Default voltage reference pin pair
  ADC->SC3    = ((0U << 3) |                         // one conversion
                 (1U << 2) |                         // Hardware average function enabled
                 (3U << 0)  );                       // 32 samples averaged
  ADC->SC1[0] = ((1U << 6) |                         // Conversion complete interrupt is enabled
                 (0x1FU  )  );
  ADC->SC1[1] =   0x1F;                              // module is disabled
  ADC->SC1[2] =   0x1F;                              // module is disabled
  ADC->SC1[3] =   0x1F;                              // module is disabled

  NVIC_EnableIRQ(ADC_IRQn);                          // enable ADC1 SEQA interrupt

  return 0;
}

/**
  \fn          int32_t ADC_Uninitialize (void)
  \brief       De-initialize Analog-to-Digital Converter
  \returns
   - \b  0: function succeeded
   - \b -1: function failed
*/
int32_t ADC_Uninitialize (void) {

  NVIC_DisableIRQ(ADC_IRQn);                         // disable ADC interrupt
  SIM->SCGC6 &= ~(1U << SIM_SCGC6_ADC_SHIFT);        // disable ADC clock

  return 0;
}

/**
  \fn          int32_t ADC_StartConversion (void)
  \brief       Start conversion
  \returns
   - \b  0: function succeeded
   - \b -1: function failed
*/
int32_t ADC_StartConversion (void) {

  ADC->SC1[0]  = (ADC->SC1[0] & ~(0x1FU)) | (19U);   // AD10 Single ended input from pin

  return 0;
}

/**
  \fn          int32_t ADC_ConversionDone (void)
  \brief       Check if conversion finished
  \returns
   - \b  0: conversion finished
   - \b -1: conversion in progress
*/
int32_t ADC_ConversionDone (void) {
  return (AD_done ? 0 : -1);
}

/**
  \fn          int32_t ADC_GetValue (void)
  \brief       Get converted value
  \returns
   - <b> >=0</b>: converted value
   - \b -1: conversion in progress or failed
*/
int32_t ADC_GetValue (void) {

  if (AD_done) {
    AD_done = 0;
    return AD_last;
  }
  return -1;
}

/**
  \fn          uint32_t ADC_GetResolution (void)
  \brief       Get resolution of Analog-to-Digital Converter
  \returns     Resolution (in bits)
*/
uint32_t ADC_GetResolution (void) {
  return ADC_RESOLUTION;
}

/**
  \fn          void ADC_IRQHandler (void)
  \brief       Analog-to-Digital Converter Interrupt Handler
*/
void ADC_IRQHandler(void) {
 uint16_t ADC_reg;

  ADC_reg = ADC->R[0];                               // read conversion result
  AD_last = ADC_reg;                                 // store converted value

  AD_done = 1;
}
